#!/usr/bin/env python

# Copyright 2015 Sam Yaple
# Copyright 2016 intel
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil
import tempfile

from yaml import dump
from yaml import safe_load
try:
    from yaml import CDumper as Dumper  # noqa: F401
    from yaml import CLoader as Loader  # noqa: F401
except ImportError:
    from yaml import Dumper  # noqa: F401
    from yaml import Loader  # noqa: F401


from ansible import constants
from ansible.plugins import action
import six

DOCUMENTATION = '''
---
module: merge_yaml
short_description: Merge yaml-style configs
description:
     - PyYAML is used to merge several yaml files into one
options:
  dest:
    description:
      - The destination file name
    required: True
    type: str
  sources:
    description:
      - A list of files on the destination node to merge together
    default: None
    required: True
    type: str
author: Sean Mooney
'''

EXAMPLES = '''
Merge multiple yaml files:

- hosts: localhost
  tasks:
    - name: Merge yaml files
      merge_yaml:
        sources:
          - "/tmp/default.yml"
          - "/tmp/override.yml"
        dest:
          - "/tmp/out.yml"
'''


class ActionModule(action.ActionBase):

    TRANSFERS_FILES = True

    def read_config(self, source):
        result = None
        # Only use config if present
        if os.access(source, os.R_OK):
            with open(source, 'r') as f:
                template_data = f.read()

            # set search path to mimic 'template' module behavior
            searchpath = [
                self._loader._basedir,
                os.path.join(self._loader._basedir, 'templates'),
                os.path.dirname(source),
            ]
            self._templar.environment.loader.searchpath = searchpath

            template_data = self._templar.template(template_data)
            result = safe_load(template_data)
        return result or {}

    def run(self, tmp=None, task_vars=None):
        if task_vars is None:
            task_vars = dict()
        result = super(ActionModule, self).run(tmp, task_vars)
        del tmp  # not used

        # save template args.
        extra_vars = self._task.args.get('vars', list())
        old_vars = self._templar._available_variables

        temp_vars = task_vars.copy()
        temp_vars.update(extra_vars)
        self._templar.set_available_variables(temp_vars)

        output = {}
        sources = self._task.args.get('sources', None)
        if not isinstance(sources, list):
            sources = [sources]
        for source in sources:
            Utils.update_nested_conf(output, self.read_config(source))

        # restore original vars
        self._templar.set_available_variables(old_vars)

        local_tempdir = tempfile.mkdtemp(dir=constants.DEFAULT_LOCAL_TMP)

        try:
            result_file = os.path.join(local_tempdir, 'source')
            with open(result_file, 'w') as f:
                f.write(dump(output, default_flow_style=False))

            new_task = self._task.copy()
            new_task.args.pop('sources', None)

            new_task.args.update(
                dict(
                    src=result_file
                )
            )

            copy_action = self._shared_loader_obj.action_loader.get(
                'copy',
                task=new_task,
                connection=self._connection,
                play_context=self._play_context,
                loader=self._loader,
                templar=self._templar,
                shared_loader_obj=self._shared_loader_obj)
            result.update(copy_action.run(task_vars=task_vars))
        finally:
            shutil.rmtree(local_tempdir)
        return result


class Utils(object):
    @staticmethod
    def update_nested_conf(conf, update):
        for k, v in six.iteritems(update):
            if isinstance(v, dict):
                conf[k] = Utils.update_nested_conf(conf.get(k, {}), v)
            else:
                conf[k] = v
        return conf



#!/usr/bin/env python

# Copyright 2018 99cloud
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import re
import subprocess  # nosec


DOCUMENTATION = '''
---
module: kolla_ceph_keyring
short_description: >
  Module for update ceph client keyring caps in kolla.
description:
  - A module used to update ceph client keyring caps in kolla.
options:
  name:
    description:
      - the client name in ceph
    required: True
    type: str
  container_name:
    description:
      - the ceph mon container name
    required: True
    default: ceph_mon
    type: str
  caps:
    description:
      - the ceph auth caps
    required: True
    type: dict
author: Jeffrey Zhang
'''

EXAMPLES = '''
- name: configure admin client caps
  kolla_ceph_keyring:
    name: client.admin
    container_name: ceph_mon
    caps:
      mds: 'allow *'
      mon: 'allow *'
      osd: 'allow *'
      mgr: 'allow *'
'''


enoent_re = re.compile(r"\bENOENT\b")


class CephKeyring(object):
    def __init__(self, name, caps, container_name='ceph_mon'):
        self.name = name
        self.caps = caps
        self.container_name = container_name
        self.changed = False
        self.message = None

    def _run(self, cmd):
        _prefix = ['docker', 'exec', self.container_name]
        cmd = _prefix + cmd
        proc = subprocess.Popen(cmd,  # nosec
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        stdout, stderr = proc.communicate()
        retcode = proc.poll()
        if retcode != 0:
            output = 'stdout: "%s", stderr: "%s"' % (stdout, stderr)
            raise subprocess.CalledProcessError(retcode, cmd, output)
        return stdout

    def _format_caps(self):
        caps = []
        for obj in sorted(self.caps):
            caps.extend([obj, self.caps[obj]])
        return caps

    def parse_stdout(self, stdout):
        keyring = json.loads(stdout)
        # there should be only one element
        return keyring[0]

    def ensure_keyring(self):
        try:
            stdout = self.get_keyring()
        except subprocess.CalledProcessError as e:
            if e.returncode != 2 or not enoent_re.search(e.output):
                # this is not a missing keyring case
                raise
            # keyring doesn't exsit, try to create it
            stdout = self.create_keyring()
            self.changed = True
            self.message = 'ceph keyring for %s is created' % self.name
        keyring = self.parse_stdout(stdout)
        if keyring['caps'] != self.caps:
            self.update_caps()
            stdout = self.get_keyring()
            keyring = self.parse_stdout(stdout)
            self.changed = True
            self.message = 'ceph keyring for %s is updated' % self.name
        self.keyring = keyring
        return self.keyring

    def get_keyring(self):
        ceph_cmd = ['ceph', '--format', 'json', 'auth', 'get', self.name]
        return self._run(ceph_cmd)

    def update_caps(self):
        ceph_cmd = ['ceph', '--format', 'json', 'auth', 'caps', self.name]
        caps = self._format_caps()
        ceph_cmd.extend(caps)
        self._run(ceph_cmd)

    def create_keyring(self):
        ceph_cmd = ['ceph', '--format', 'json', 'auth',
                    'get-or-create', self.name]
        caps = self._format_caps()
        ceph_cmd.extend(caps)
        return self._run(ceph_cmd)


def main():
    specs = dict(
        name=dict(type='str', required=True),
        container_name=dict(type='str', default='ceph_mon'),
        caps=dict(type='dict', required=True)
    )
    module = AnsibleModule(argument_spec=specs)  # noqa
    params = module.params
    ceph_keyring = CephKeyring(params['name'],
                               params['caps'],
                               params['container_name'])
    try:
        keyring = ceph_keyring.ensure_keyring()
        module.exit_json(changed=ceph_keyring.changed,
                         keyring=keyring,
                         message=ceph_keyring.message)
    except subprocess.CalledProcessError as ex:
        msg = ('Failed to call command: %s returncode: %s output: %s' %
               (ex.cmd, ex.returncode, ex.output))
        module.fail_json(msg=msg)


from ansible.module_utils.basic import *  # noqa
if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import sys
import yaml


def mergepwd(old, new, final):
    with open(old, "r") as old_file:
        old_passwords = yaml.safe_load(old_file)

    with open(new, "r") as new_file:
        new_passwords = yaml.safe_load(new_file)

    if not isinstance(old_passwords, dict):
        print("ERROR: Old passwords file not in expected key/value format")
        sys.exit(1)

    if not isinstance(new_passwords, dict):
        print("ERROR: New passwords file not in expected key/value format")
        sys.exit(1)

    new_passwords.update(old_passwords)

    with open(final, "w") as destination:
        yaml.safe_dump(new_passwords, destination, default_flow_style=False)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--old", help="old password file", required=True)
    parser.add_argument("--new", help="new password file", required=True)
    parser.add_argument("--final", help="merged password file", required=True)
    args = parser.parse_args()
    mergepwd(args.old, args.new, args.final)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This module creates a list of cron intervals for a node in a group of nodes
# to ensure each node runs a cron in round robbin style.

from __future__ import print_function
import argparse
import json
import sys

MINUTE_SPAN = 1
HOUR_SPAN = 60
DAY_SPAN = 24 * HOUR_SPAN
WEEK_SPAN = 7 * DAY_SPAN


class RotationIntervalTooLong(Exception):
    pass


def json_exit(msg=None, failed=False, changed=False):
    if type(msg) is not dict:
        msg = {'msg': str(msg)}
    msg.update({'failed': failed, 'changed': changed})
    print(json.dumps(msg))
    sys.exit()


def generate(host_index, total_hosts, total_rotation_mins):
    min = '*'  # 0-59
    hour = '*'  # 0-23
    day = '*'  # 0-6 (day of week)
    crons = []

    if host_index >= total_hosts:
        return crons

    # We need to rotate the key every total_rotation_mins minutes.
    # When there are N hosts, each host should rotate once every N *
    # total_rotation_mins minutes, in a round-robin manner.
    # We can generate a cycle for index 0, then add an offset specific to each
    # host.
    # NOTE: Minor under-rotation is better than over-rotation since tokens
    # may become invalid if keys are over-rotated.
    host_rotation_mins = total_rotation_mins * total_hosts
    host_rotation_offset = total_rotation_mins * host_index

    # Can't currently rotate less than once per week.
    if total_rotation_mins > WEEK_SPAN:
        msg = ("Unable to schedule fernet key rotation with an interval "
               "greater than 1 week divided by the number of hosts")
        raise RotationIntervalTooLong(msg)

    # Build crons multiple of a day
    elif host_rotation_mins > DAY_SPAN:
        time = host_rotation_offset
        while time + total_rotation_mins <= WEEK_SPAN:
            day = time // DAY_SPAN
            hour = time % HOUR_SPAN
            min = time % HOUR_SPAN
            crons.append({'min': min, 'hour': hour, 'day': day})

            time += host_rotation_mins

    # Build crons for multiple of an hour
    elif host_rotation_mins > HOUR_SPAN:
        time = host_rotation_offset
        while time + total_rotation_mins <= DAY_SPAN:
            hour = time // HOUR_SPAN
            min = time % HOUR_SPAN
            crons.append({'min': min, 'hour': hour, 'day': day})

            time += host_rotation_mins

    # Build crons for multiple of a minute
    else:
        time = host_rotation_offset
        while time + total_rotation_mins <= HOUR_SPAN:
            min = time // MINUTE_SPAN
            crons.append({'min': min, 'hour': hour, 'day': day})

            time += host_rotation_mins

    return crons


def main():
    parser = argparse.ArgumentParser(description='''Creates a list of cron
        intervals for a node in a group of nodes to ensure each node runs
        a cron in round robin style.''')
    parser.add_argument('-t', '--time',
                        help='Time in minutes for a key rotation cycle',
                        required=True,
                        type=int)
    parser.add_argument('-i', '--index',
                        help='Index of host starting from 0',
                        required=True,
                        type=int)
    parser.add_argument('-n', '--number',
                        help='Number of hosts',
                        required=True,
                        type=int)
    args = parser.parse_args()
    try:
        jobs = generate(args.index, args.number, args.time)
    except Exception as e:
        json_exit(str(e), failed=True)
    json_exit({'cron_jobs': jobs})


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Copyright 2016 99cloud Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import docker
import json
import re

from ansible.module_utils.basic import AnsibleModule

DOCUMENTATION = '''
---
module: kolla_toolbox
short_description: >
  Module for invoking ansible module in kolla_toolbox container.
description:
  - A module targerting at invoking ansible module in kolla_toolbox
    container as used by Kolla project.
options:
  module_name:
    description:
      - The module name to invoke
    required: True
    type: str
  module_args:
    description:
      - The module args use by the module
    required: False
    type: str or dict
  module_extra_vars:
    description:
      - The extra variables used by the module
    required: False
    type: str or dict
  api_version:
    description:
      - The version of the API for docker-py to use when contacting Docker
    required: False
    type: str
    default: auto
  timeout:
    description:
      - The default timeout for docker-py client when contacting Docker API
    required: False
    type: int
    default: 180
author: Jeffrey Zhang
'''

EXAMPLES = '''
- hosts: controller
  tasks:
    - name: Ensure the direct absent
      kolla_toolbox:
        module_name: file
        module_args: path=/tmp/a state=absent
    - name: Create mysql database
      kolla_toolbox:
        module_name: mysql_db
        module_args:
          login_host: 192.168.1.10
          login_user: root
          login_password: admin
          name: testdb
    - name: Creating default user role
      kolla_toolbox:
        module_name: os_keystone_role
        module_args:
          name: _member_
          auth: "{{ '{{ openstack_keystone_auth }}' }}"
        module_extra_vars:
          openstack_keystone_auth:
            auth_url: http://127.0.0.1:5000
            username: admin
            password: password
            project_name: "admin"
            domain_name: "default"
'''


JSON_REG = re.compile('^(?P<host>\w+) \| (?P<status>\w+)!? =>(?P<stdout>.*)$',
                      re.MULTILINE | re.DOTALL)
NON_JSON_REG = re.compile(('^(?P<host>\w+) \| (?P<status>\w+)!? \| '
                           'rc=(?P<exit_code>\d+) >>\n(?P<stdout>.*)\n$'),
                          re.MULTILINE | re.DOTALL)


def gen_commandline(params):
    command = ['ansible', 'localhost']
    if params.get('module_name'):
        command.extend(['-m', params.get('module_name')])
    if params.get('module_args'):
        module_args = params.get('module_args')
        if isinstance(module_args, dict):
            module_args = ' '.join("{}='{}'".format(key, value)
                                   for key, value in module_args.items())
        command.extend(['-a', module_args])
    if params.get('module_extra_vars'):
        extra_vars = params.get('module_extra_vars')
        if isinstance(extra_vars, dict):
            extra_vars = json.dumps(extra_vars)
        command.extend(['--extra-vars', extra_vars])
    return command


def get_docker_client():
    return docker.APIClient


def main():
    specs = dict(
        module_name=dict(required=True, type='str'),
        module_args=dict(type='str'),
        module_extra_vars=dict(type='json'),
        api_version=dict(required=False, type='str', default='auto'),
        timeout=dict(required=False, type='int', default=180),
    )
    module = AnsibleModule(argument_spec=specs, bypass_checks=True)
    client = get_docker_client()(
        version=module.params.get('api_version'),
        timeout=module.params.get('timeout'))
    command_line = gen_commandline(module.params)
    kolla_toolbox = client.containers(filters=dict(name='kolla_toolbox',
                                                   status='running'))
    if not kolla_toolbox:
        module.fail_json(msg='kolla_toolbox container is not running.')

    kolla_toolbox = kolla_toolbox[0]
    job = client.exec_create(kolla_toolbox, command_line)
    output = client.exec_start(job)

    for exp in [JSON_REG, NON_JSON_REG]:
        m = exp.match(output)
        if m:
            inner_output = m.groupdict().get('stdout')
            status = m.groupdict().get('status')
            break
    else:
        module.fail_json(
            msg='Can not parse the inner module output: %s' % output)

    ret = dict()
    try:
        ret = json.loads(inner_output)
    except ValueError:
        # Some modules (e.g. command) do not produce a JSON output. Instead,
        # check the status, and assume changed on success.
        ret['stdout'] = inner_output
        if status != "SUCCESS":
            ret['failed'] = True
        else:
            # No way to know whether changed - assume yes.
            ret['changed'] = True

    module.exit_json(**ret)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Copyright 2016 99cloud
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import docker

from ansible.module_utils.basic import AnsibleModule

DOCUMENTATION = '''
---
module: kolla_container_facts
short_description: Module for collecting Docker container facts
description:
  - A module targeting at collecting Docker container facts. It is used for
    detecting whether the container is running on host in Kolla.
options:
  api_version:
    description:
      - The version of the api for docker-py to use when contacting docker
    required: False
    type: str
    default: auto
  name:
    description:
      - Name or names of the containers
    required: False
    type: str or list
author: Jeffrey Zhang
'''

EXAMPLES = '''
- hosts: all
  tasks:
    - name: Gather docker facts
      kolla_container_facts:

    - name: Gather glance container facts
      kolla_container_facts:
        name:
          - glance_api
          - glance_registry
'''


def get_docker_client():
    return docker.APIClient


def main():
    argument_spec = dict(
        name=dict(required=False, type='list', default=[]),
        api_version=dict(required=False, type='str', default='auto')
    )

    module = AnsibleModule(argument_spec=argument_spec)

    results = dict(changed=False, _containers=[])
    client = get_docker_client()(version=module.params.get('api_version'))
    containers = client.containers()
    names = module.params.get('name')
    if names and not isinstance(names, list):
        names = [names]
    for container in containers:
        for container_name in container['Names']:
            # remove '/' prefix character
            container_name = container_name[1:]
            if names and container_name not in names:
                continue
            results['_containers'].append(container)
            results[container_name] = container
    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import fnmatch
import json
import logging
import os
import re
import sys

import jinja2
import yaml


PROJECT_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))

NEWLINE_EOF_INCLUDE_PATTERNS = ['*.j2', '*.yml', '*.py', '*.sh']
NEWLINE_EOF_EXCLUDE_PATTERNS = ['.tox', '.testrepository', '.git']

# Render json file by using jinja2 template is OK
JSON_J2_INCLUDE_PATTERNS = ['*.json.j2', '*.json']
JSON_J2_EXCLUDE_PATTERNS = ['.tox', '.testrepository', '.git']

YAML_INCLUDE_PATTERNS = ['*.yml']
YAML_EXCLUDE_PATTERNS = ['.tox', '.testrepository', '.git',
                         'defaults', 'templates', 'vars']

logging.basicConfig()
LOG = logging.getLogger(__name__)


def check_newline_eof():
    includes = r'|'.join([fnmatch.translate(x)
                          for x in NEWLINE_EOF_INCLUDE_PATTERNS])
    excludes = r'|'.join([fnmatch.translate(x)
                          for x in NEWLINE_EOF_EXCLUDE_PATTERNS])
    return_code = 0

    def has_newline_eof(path):
        with open(path, 'r') as f:
            data = f.read()
            if data and data[-1] != '\n':
                LOG.error('%s file error: no newline at end of file', path)
                return False
        return True

    for root, dirs, files in os.walk(PROJECT_ROOT):
        dirs[:] = [d for d in dirs if not re.match(excludes, d)]
        for f in files:
            if not re.match(excludes, f) and re.match(includes, f):
                if not has_newline_eof(os.path.join(root, f)):
                    return_code = 1
    return return_code


def check_json_j2():
    includes = r'|'.join([fnmatch.translate(x)
                          for x in JSON_J2_INCLUDE_PATTERNS])
    excludes = r'|'.join([fnmatch.translate(x)
                          for x in JSON_J2_EXCLUDE_PATTERNS])
    return_code = 0

    def bool_filter(value):
        return True

    def basename_filter(text):
        return text.split('\\')[-1]

    # Mock ansible hostvars variable, which is a nested dict
    def hostvars():
        return collections.defaultdict(hostvars)

    # Mock Ansible groups variable, which is a dict of lists.
    def groups():
        return collections.defaultdict(list)

    def validate_json_j2(root, filename):
        env = jinja2.Environment(  # nosec: not used to render HTML
            loader=jinja2.FileSystemLoader(root))
        env.filters['bool'] = bool_filter
        env.filters['basename'] = basename_filter
        template = env.get_template(filename)
        # Mock ansible variables.
        context = {
            'hostvars': hostvars(),
            'groups': groups(),
            'cluster_interface': 'cluster_interface',
            'storage_interface': 'storage_interface',
            'inventory_hostname': 'hostname'
        }
        data = template.render(**context)
        json.loads(data)
    for root, dirs, files in os.walk(PROJECT_ROOT):
        dirs[:] = [d for d in dirs if not re.match(excludes, d)]
        for filename in files:
            if not re.match(excludes, filename) and \
                    re.match(includes, filename):
                fullpath = os.path.join(root, filename)
                try:
                    validate_json_j2(root, filename)
                except (ValueError, jinja2.exceptions.TemplateError):
                    return_code = 1
                    LOG.exception('%s file error', fullpath)
    return return_code


def check_docker_become():
    """All tasks that use Docker should have 'become: true'."""
    includes = r'|'.join([fnmatch.translate(x)
                          for x in YAML_INCLUDE_PATTERNS])
    excludes = r'|'.join([fnmatch.translate(x)
                          for x in YAML_EXCLUDE_PATTERNS])
    docker_modules = ('kolla_docker', 'kolla_ceph_keyring',
                      'kolla_container_facts', 'kolla_toolbox')
    cmd_modules = ('command', 'shell')
    return_code = 0
    roles_path = os.path.join(PROJECT_ROOT, 'ansible', 'roles')
    for root, dirs, files in os.walk(roles_path):
        dirs[:] = [d for d in dirs if not re.match(excludes, d)]
        for filename in files:
            if not re.match(excludes, filename) and \
                    re.match(includes, filename):
                fullpath = os.path.join(root, filename)
                with open(fullpath) as fp:
                    tasks = yaml.safe_load(fp)
                tasks = tasks or []
                for task in tasks:
                    for module in docker_modules:
                        if module in task and not task.get('become'):
                            return_code = 1
                            LOG.error("Use of %s module without become in "
                                      "task %s in %s",
                                      module, task['name'], fullpath)
                    for module in cmd_modules:
                        if (module in task and
                                task[module].startswith('docker') and
                                not task.get('become')):
                            return_code = 1
                            LOG.error("Use of docker in %s module without "
                                      "become in task %s in %s",
                                      module, task['name'], fullpath)

    return return_code


def main():
    checks = (
        check_newline_eof,
        check_json_j2,
        check_docker_become,
    )
    return sum([check() for check in checks])

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import imp
import os
import sys

from oslotest import base

this_dir = os.path.dirname(sys.modules[__name__].__file__)
file_under_test = os.path.join(this_dir, '..', 'ansible',
                               'roles', 'keystone', 'files',
                               'fernet_rotate_cron_generator.py')
generator = imp.load_source('generator', file_under_test)


class FernetRotateCronGeneratorTest(base.BaseTestCase):

    def _test(self, host_index, total_hosts, total_rotation_mins, expected):
        result = generator.generate(host_index, total_hosts,
                                    total_rotation_mins)
        self.assertEqual(expected, result)

    def test_invalid_index(self):
        expected = []
        self._test(1, 0, 0, expected)

    # Invalid: total_rotation_mins > WEEK_SPAN:

    def test_1_week_1_min_1_host(self):
        self.assertRaises(generator.RotationIntervalTooLong,
                          generator.generate,
                          0, 1, 7 * 24 * 60 + 1)

    def test_1_week_1_min_2_hosts(self):
        self.assertRaises(generator.RotationIntervalTooLong,
                          generator.generate,
                          0, 2, 7 * 24 * 60 + 1)

    # host_rotation_mins > DAY_SPAN:

    def test_1_day_2_hosts(self):
        expected = [{"min": 0, "hour": 0, "day": day}
                    for day in range(0, 7, 2)]
        self._test(0, 2, 24 * 60, expected)

        expected = [{"min": 0, "hour": 0, "day": day}
                    for day in range(1, 7, 2)]
        self._test(1, 2, 24 * 60, expected)

    def test_1_day_3_hosts(self):
        expected = [{"min": 0, "hour": 0, "day": day}
                    for day in range(0, 7, 3)]
        self._test(0, 3, 24 * 60, expected)

        expected = [{"min": 0, "hour": 0, "day": day}
                    for day in range(1, 7, 3)]
        self._test(1, 3, 24 * 60, expected)

        expected = [{"min": 0, "hour": 0, "day": day}
                    for day in range(2, 7, 3)]
        self._test(2, 3, 24 * 60, expected)

    def test_2_days_1_host(self):
        expected = [{"min": 0, "hour": 0, "day": day}
                    for day in range(0, 6, 2)]
        self._test(0, 1, 2 * 24 * 60, expected)

    def test_2_days_2_hosts(self):
        expected = [{"min": 0, "hour": 0, "day": day}
                    for day in range(0, 7, 4)]
        self._test(0, 2, 2 * 24 * 60, expected)

        expected = [{"min": 0, "hour": 0, "day": 2}]
        self._test(1, 2, 2 * 24 * 60, expected)

    def test_3_days_1_host(self):
        # NOTE: This is the default config in kolla-ansible for 1 host.
        expected = [{"min": 0, "hour": 0, "day": day}
                    for day in range(0, 6, 3)]
        self._test(0, 1, 3 * 24 * 60, expected)

    def test_3_days_3_hosts(self):
        # NOTE: This is the default config in kolla-ansible for 3 hosts.
        expected = [{"min": 0, "hour": 0, "day": 0}]
        self._test(0, 3, 3 * 24 * 60, expected)

        expected = [{"min": 0, "hour": 0, "day": 3}]
        self._test(1, 3, 3 * 24 * 60, expected)

        expected = []
        self._test(2, 3, 3 * 24 * 60, expected)

    def test_1_week_1_host(self):
        expected = [{"min": 0, "hour": 0, "day": 0}]
        self._test(0, 1, 7 * 24 * 60, expected)

    # total_rotation_mins > HOUR_SPAN:

    def test_1_hour_2_hosts(self):
        expected = [{"min": 0, "hour": hour, "day": "*"}
                    for hour in range(0, 24, 2)]
        self._test(0, 2, 60, expected)

        expected = [{"min": 0, "hour": hour, "day": "*"}
                    for hour in range(1, 24, 2)]
        self._test(1, 2, 60, expected)

    def test_2_hours_1_host(self):
        expected = [{"min": 0, "hour": hour, "day": "*"}
                    for hour in range(0, 24, 2)]
        self._test(0, 1, 2 * 60, expected)

    def test_2_hours_2_hosts(self):
        expected = [{"min": 0, "hour": hour, "day": "*"}
                    for hour in range(0, 24, 4)]
        self._test(0, 2, 2 * 60, expected)

        expected = [{"min": 0, "hour": hour, "day": "*"}
                    for hour in range(2, 24, 4)]
        self._test(1, 2, 2 * 60, expected)

    def test_61_minutes_1_host(self):
        # FIXME: Anything greater than 1 hour (unless an integer number of
        # hours) returns no jobs.
        expected = [{"min": hour, "hour": hour, "day": "*"}
                    for hour in range(0, 23)]
        self._test(0, 1, 61, expected)

    def test_61_minutes_2_hosts(self):
        # FIXME: Anything greater than 1 hour (unless an integer number of
        # hours) returns no jobs.
        expected = [{"min": hour, "hour": hour, "day": "*"}
                    for hour in range(0, 24, 2)]
        self._test(0, 2, 61, expected)

        expected = [{"min": hour, "hour": hour, "day": "*"}
                    for hour in range(1, 23, 2)]
        self._test(1, 2, 61, expected)

    def test_1_day_1_host(self):
        expected = [{"min": 0, "hour": 0, "day": "*"}]
        self._test(0, 1, 24 * 60, expected)

    def test_12_hours_2_hosts(self):
        expected = [{"min": 0, "hour": 0, "day": "*"}]
        self._test(0, 2, 12 * 60, expected)

        expected = [{"min": 0, "hour": 12, "day": "*"}]
        self._test(1, 2, 12 * 60, expected)

    # else:

    def test_1_minute_1_host(self):
        expected = [{"min": min, "hour": "*", "day": "*"}
                    for min in range(60)]
        self._test(0, 1, 1, expected)

    def test_1_minute_2_hosts(self):
        expected = [{"min": min, "hour": "*", "day": "*"}
                    for min in range(0, 60, 2)]
        self._test(0, 2, 1, expected)

        expected = [{"min": min, "hour": "*", "day": "*"}
                    for min in range(1, 60, 2)]
        self._test(1, 2, 1, expected)

    def test_2_minutes_1_host(self):
        expected = [{"min": min, "hour": "*", "day": "*"}
                    for min in range(0, 60, 2)]
        self._test(0, 1, 2, expected)

    def test_2_minutes_2_hosts(self):
        expected = [{"min": min, "hour": "*", "day": "*"}
                    for min in range(0, 60, 4)]
        self._test(0, 2, 2, expected)

        expected = [{"min": min, "hour": "*", "day": "*"}
                    for min in range(2, 60, 4)]
        self._test(1, 2, 2, expected)

    def test_1_hour_1_host(self):
        expected = [{"min": 0, "hour": "*", "day": "*"}]
        self._test(0, 1, 60, expected)
